# Discussion

## Pros and cons

Pros:

- It is a software development kit dedicated to nodejs/javascript developers.
- Very fast run-time (especially using NaifJs as embedded library)
- Well matches with stateful-based, rule-based applications as task-based workflows.
  E.g. Games, interactive stories, real-time/embedded (voice) assistants/cobots.
- Good for slot-filling / complex slot validation, closed-domain / task completion dialogues.
- No dependencies from external (cloud-based) "NLU" platforms (if regexp pattern matching is acceptable).
- NaifJs command line utilities help to quickly test dialogues from command line and from a Telegram bot.
- No machine-learning magics.

Cons:

- NaifJs is a low-level framework, dedicated to nodejs/javascript developers.
- Designer/coder must think, design and implement dialogues as "state machines". 
- Not a valid approach for FAQ-like question-answering or open-domain/chit-chat chatbots. 
- No machine-learning magics.


## BackStory

Few years ago, I started working in the idea to implement a human-machine multi-turn conversation using the conceptual model of a state machine.
At time, I was focused on how to implement a conversational agent (chatbot) for accomplish workflows/task in e-commerce/e-shopping domain. 
In general I was focused to realize a simple tool for developers allowing conversational, closed-domain, task-oriented workflows.

In 2016 I realized first prototype in Ruby language 
(slides of my talk: [Naif — Ruby micro framework to build dumb chat machines](https://convcomp.it/naif-ruby-micro-framework-to-build-dumb-chat-machines-5c552a8c8f7e#.exp91nt72)). The reason of name *Naif* was because I was aware that the state machine approach do not pretend to be a magic "conversational AI" solution.


As researcher at the Italian National Research Council Institute for Educational Technology (ITD-CNR), 
in 2019 I ported the original prototype from Ruby to NodeJs, building NaifJs as it is now. 
I used NaifJs as multi-user dialog manager engine of [CPIAbot](https://www.itd.cnr.it/Progetti_Rispo1.php?PROGETTO=1193), 
a Telegram voice/multi-modal chatbot to help foreigners refugees, students of Italian public adult schools, in learning Italian language L2.

Related academic papers (introducing NaifJs usage):

- F. Ravicchio, G. Robino, S. Torsani.
  *Un assistente conversazionale a supporto dell’apprendimento dell’italiano L2 per migranti: CPIAbot*. Sep 2020. 
  Published in Italian Journal of Educational Technology (IJET). [PDF](https://ijet.itd.cnr.it/article/view/1142/1081) https://doi.org/10.17471/2499-4324/1142

- F. Ravicchio, G. Robino, G. Trentin.
  *CPIAbot: un chatbot nell’insegnamento dell’Italiano L2 per stranieri*. 
  Published in Didamatica 2019 acts, Best Paper Award in section: BYOD. Mobile e Mixed Learning (ISBN 978-88-98091-50-8) [PDF](https://www.aicanet.it/didamatica2019/atti-2019) p. 77-86.


## Outcomes

What seems to me a still valid in NaifJs is the implementation of a multi-turn dialog 
as a graph of function handlers that represents states of a state machine. 

Naif hard-codes workflows into javascript code and that's good. 

I imagine NaifJs dialog units as an "assembler" language, 
acting as the *transpilation* of a possible high-level common programming language 
(allowing conversational designers to quick realize chatbots). 


---

[top](#) | [home](../README.md)
