# To do 

## Architectural improvements

- [ ] 🛠 stateid + userid
 
  the common storage is a bad architecture choice 
  (possible bugs in time delayed transactions, in a multiuser scenario). 
  It must be replaced asap.

- [ ] synchronous/asynchronous state handlers
 
  A special note concerns the synchronous/asynchronous behaviors of nodejs. 
  Currently input and output state handlers are normal "synchronous" javascript functions. 
  That's debatable. 

  > ⚠️  WARNING: when NaifJs run as a server in a multi user channel, 
  > in case of delayed/promised/timeout functions 
  > (anything that will run out-of-time in a future (~promise), 
  > please remember to pass the *session id* (*user id*) to functions that will be activated with a time delay.

- [ ] sessions database
 
  Currently the sessions are managed as in-memory hash table (a javascript object), 
  read from disk at up time and persisted to disk on down time.
  A better choice is to decouple sessions into a separate database (e.g. using REDIS, or LMDB, or any other DB).

## APIs

- [ ] add a default timeout handler policy

  NaifJs could supply a ready-made default response timeout handler, asking every M seconds the last prompt P. 

- [ ] Prompts timeout data management 
 
  So far, timeout handlers are managed in a dedicated in-memory object. 
  Not a perfect choice. Maybe would be part of the session object.


## 💡 Debugging features (in `naif test`) 

the `naif test` command line program, and the `naif telegram` could be enabled by inline commands for debug purposes,
maybe 

- using a special prefix e.g. `/command` `:command`, 
- On `return` keystroke (possible on CLI, not possible on Telegram) 
 
to show, by example, these info:

 - stateid
 - the dialog flow stateids stack [state1 -> state2-> state3 -> etc.] 
 - dialog unit variables 
 - all session data


## 💡 nodes introspection

The idea is that each input and output nodes could be queried (for debugging / reporting purposes).
In terms of javascript, it means just adding attributes to a state function handler:

```javascript
// someDialogUnit.js
function someInputState() { /// ... }
function someOutputState() { /// ... }

someInputState.type = 'input'
someInputState.description = 'bla bla bla...'

someOutputState.type = 'output'
someOutputState.description = 'blablabla blabla bla...'
```

Introspection "metadata" could be generated by a `naif generate` JSON input configuration. 


## 💡 Graphic representation of a state-machine

- In output

  think about how generate (in real-time) a graphic representation of the state machine path flow. 

- In input

  it could be useful to generate state machine code, starting from a graphical representation of a state machine. 


## 💡 Scripting language transpilation
 
How to create a dialogue javascript source code from a JSON file or better, 
a text script with a special indentation and meta-tags (like the .script). And viceversa.


## Other things 

- [ ] Add more meaningful/complete application examples 

  e.g. task completion, see *promemoria*, slot-filling, see *conosciamoci*, etc.

- [ ] Regexp pattern matching implementation must be rethought. 

- [ ] 💡 Enable NaifJs run-time to run on a browser

  Session file must be substituted with a in-memory storage.

- [ ] Log management 
 
  currently optional for `naif test` and  `naif telegram` is ugly implemented. 
  It must be fixed.

- [ ] `naif server`

  New feature: the idea is to wrap NaifJs engine behind a HTTP/TCP server.


---

[top](#) | [home](../README.md)

